"""Interface to Plotly's /v2/plot-schema endpoints."""
from __future__ import absolute_import

from chart_studio.api.v2.utils import build_url, make_params, request

RESOURCE = "plot-schema"


def retrieve(sha1, **kwargs):
    """
    Retrieve the most up-to-date copy of the plot-schema wrt the given hash.

    :param (str) sha1: The last-known hash of the plot-schema (or '').
    :returns: (requests.Response) Returns response directly from requests.

    """
    url = build_url(RESOURCE)
    params = make_params(sha1=sha1)
    return request("get", url, params=params, **kwargs)



"""Interface to Plotly's /v2/images endpoints."""
from __future__ import absolute_import

from chart_studio.api.v2.utils import build_url, request

RESOURCE = "images"


def create(body):
    """
    Generate an image (which does not get saved on Plotly).

    :param (dict) body: A mapping of body param names to values.
    :returns: (requests.Response) Returns response directly from requests.

    """
    url = build_url(RESOURCE)
    return request("post", url, json=body)



from base64 import b64encode

from requests.compat import builtin_str, is_py2


def _to_native_string(string, encoding):
    if isinstance(string, builtin_str):
        return string
    if is_py2:
        return string.encode(encoding)
    return string.decode(encoding)


def to_native_utf8_string(string):
    return _to_native_string(string, "utf-8")


def to_native_ascii_string(string):
    return _to_native_string(string, "ascii")


def basic_auth(username, password):
    """
    Creates the basic auth value to be used in an authorization header.

    This is mostly copied from the requests library.

    :param (str) username: A Plotly username.
    :param (str) password: The password for the given Plotly username.
    :returns: (str) An 'authorization' header for use in a request header.

    """
    if isinstance(username, str):
        username = username.encode("latin1")

    if isinstance(password, str):
        password = password.encode("latin1")

    return "Basic " + to_native_ascii_string(
        b64encode(b":".join((username, password))).strip()
    )



from __future__ import absolute_import

from chart_studio.api.v2 import (
    dash_apps,
    dashboards,
    files,
    folders,
    grids,
    images,
    plot_schema,
    plots,
    spectacle_presentations,
    users,
)



"""
Interface to Plotly's /v2/dashboards endpoints.

Partially complete at the moment. Only being used by
plotly.plotly.dashboard_ops.
"""
from __future__ import absolute_import

from chart_studio.api.v2.utils import build_url, request

RESOURCE = "dashboards"


def create(body):
    """Create a dashboard."""
    url = build_url(RESOURCE)
    return request("post", url, json=body)


def list():
    """Returns the list of all users' dashboards."""
    url = build_url(RESOURCE)
    return request("get", url)


def retrieve(fid):
    """Retrieve a dashboard from Plotly."""
    url = build_url(RESOURCE, id=fid)
    return request("get", url)


def update(fid, content):
    """Completely update the writable."""
    url = build_url(RESOURCE, id=fid)
    return request("put", url, json=content)


def schema():
    """Retrieve the dashboard schema."""
    url = build_url(RESOURCE, route="schema")
    return request("get", url)



from setuptools import setup
import os


def readme():
    parent_dir = os.path.dirname(os.path.realpath(__file__))
    with open(os.path.join(parent_dir, "README.md")) as f:
        return f.read()


setup(
    name="chart-studio-figlinq",
    version="0.1.9",
    author="figlinq.com",
    author_email="info@figlinq.com",
    maintainer="figlinq.com",
    maintainer_email="info@figlinq.com",
    url="https://github.com/figlinq/chart-studio-figlinq",
    project_urls={
        "Github": "https://github.com/figlinq/chart-studio-figlinq",
    },
    description="Utilities for interfacing with Plotly's Chart Studio and figlinq.com",
    long_description=readme(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Scientific/Engineering :: Visualization",
    ],
    license="MIT",
    packages=[
        "chart_studio",
        "chart_studio.api",
        "chart_studio.api.v2",
        "chart_studio.dashboard_objs",
        "chart_studio.grid_objs",
        "chart_studio.plotly",
        "chart_studio.plotly.chunked_requests",
        "chart_studio.presentation_objs",
    ],
    package_data={
        "chart_studio": ["templates/*.json"],  # Include JSON files in the "data" folder
    },
    include_package_data=True,
    install_requires=[
        "plotly>=6.0.1",
        "requests>=2.30.0",
        "retrying>=1.3.3",
        "six",
    ],
    zip_safe=False,
)



"""
utils
=====

Low-level functionality NOT intended for users to EVER use.

"""
from __future__ import absolute_import

import os.path
import re
import threading
import warnings

import json as _json

from _plotly_utils.exceptions import PlotlyError
from _plotly_utils.optional_imports import get_module

# Optional imports, may be None for users that only use our core functionality.
numpy = get_module("numpy")
pandas = get_module("pandas")
sage_all = get_module("sage.all")


### incase people are using threading, we lock file reads
lock = threading.Lock()


http_msg = (
    "The plotly_domain and plotly_api_domain of your config file must start "
    "with 'https', not 'http'. If you are not using On-Premise then run the "
    "following code to ensure your plotly_domain and plotly_api_domain start "
    "with 'https':\n\n\n"
    "import plotly\n"
    "plotly.tools.set_config_file(\n"
    "    plotly_domain='https://plotly.com',\n"
    "    plotly_api_domain='https://api.plotly.com'\n"
    ")\n\n\n"
    "If you are using On-Premise then you will need to use your company's "
    "domain and api_domain urls:\n\n\n"
    "import plotly\n"
    "plotly.tools.set_config_file(\n"
    "    plotly_domain='https://plotly.your-company.com',\n"
    "    plotly_api_domain='https://plotly.your-company.com'\n"
    ")\n\n\n"
    "Make sure to replace `your-company.com` with the URL of your Plotly "
    "On-Premise server.\nSee "
    "https://plotly.com/python/getting-started/#special-instructions-for-plotly-onpremise-users "
    "for more help with getting started with On-Premise."
)


### general file setup tools ###


def load_json_dict(filename, *args):
    """Checks if file exists. Returns {} if something fails."""
    data = {}
    if os.path.exists(filename):
        lock.acquire()
        with open(filename, "r") as f:
            try:
                data = _json.load(f)
                if not isinstance(data, dict):
                    data = {}
            except:
                data = {}  # TODO: issue a warning and bubble it up
        lock.release()
        if args:
            return {key: data[key] for key in args if key in data}
    return data


def save_json_dict(filename, json_dict):
    """Save json to file. Error if path DNE, not a dict, or invalid json."""
    if isinstance(json_dict, dict):
        # this will raise a TypeError if something goes wrong
        json_string = _json.dumps(json_dict, indent=4)
        lock.acquire()
        with open(filename, "w") as f:
            f.write(json_string)
        lock.release()
    else:
        raise TypeError("json_dict was not a dictionary. not saving.")


def ensure_file_exists(filename):
    """Given a valid filename, make sure it exists (will create if DNE)."""
    if not os.path.exists(filename):
        head, tail = os.path.split(filename)
        ensure_dir_exists(head)
        with open(filename, "w") as f:
            pass  # just create the file


def ensure_dir_exists(directory):
    """Given a valid directory path, make sure it exists."""
    if dir:
        if not os.path.isdir(directory):
            os.makedirs(directory)


def get_first_duplicate(items):
    seen = set()
    for item in items:
        if item not in seen:
            seen.add(item)
        else:
            return item
    return None


### source key
def is_source_key(key):
    src_regex = re.compile(r".+src$")
    if src_regex.match(key) is not None:
        return True
    else:
        return False


### validation
def validate_world_readable_and_sharing_settings(option_set):
    if (
        "world_readable" in option_set
        and option_set["world_readable"] is True
        and "sharing" in option_set
        and option_set["sharing"] is not None
        and option_set["sharing"] != "public"
    ):
        raise PlotlyError(
            "Looks like you are setting your plot privacy to both "
            "public and private.\n If you set world_readable as True, "
            "sharing can only be set to 'public'"
        )
    elif (
        "world_readable" in option_set
        and option_set["world_readable"] is False
        and "sharing" in option_set
        and option_set["sharing"] == "public"
    ):
        raise PlotlyError(
            "Looks like you are setting your plot privacy to both "
            "public and private.\n If you set world_readable as "
            "False, sharing can only be set to 'private' or 'secret'"
        )
    elif "sharing" in option_set and option_set["sharing"] not in [
        "public",
        "private",
        "secret",
        None,
    ]:
        raise PlotlyError(
            "The 'sharing' argument only accepts one of the following "
            "strings:\n'public' -- for public plots\n"
            "'private' -- for private plots\n"
            "'secret' -- for private plots that can be shared with a "
            "secret url"
        )


def validate_plotly_domains(option_set):
    domains_not_none = []
    for d in ["plotly_domain", "plotly_api_domain"]:
        if d in option_set and option_set[d]:
            domains_not_none.append(option_set[d])

    if not all(d.lower().startswith("https") for d in domains_not_none):
        warnings.warn(http_msg, category=UserWarning)


def set_sharing_and_world_readable(option_set):
    if "world_readable" in option_set and "sharing" not in option_set:
        option_set["sharing"] = "public" if option_set["world_readable"] else "private"

    elif "sharing" in option_set and "world_readable" not in option_set:
        if option_set["sharing"] == "public":
            option_set["world_readable"] = True
        else:
            option_set["world_readable"] = False


def validate_fid(test):
    """
    Validate a fid string in the format `username:idlocal`.

    :param test: The `username:idlocal` pair.
    :return: True if valid, False otherwise.
    :rtype: bool

    """

    USERNAME = r'[.a-zA-Z0-9_-]+'
    IDLOCAL = r'(?:-[1-2]|[0-9]|[1-9]\d+)'
    FID = fr'^({USERNAME}):({IDLOCAL})$'

    if isinstance(test, str) and re.match(FID, test):
        return True
    return False